@@ -304,7 +304,7 @@ string PrintForwardDeclare(const NamedDecl* decl,
       // A local class (class defined inside a function).
       fwd_decl = std::string(fn->getName()) + "::" + fwd_decl;
     } else {
-      CHECK_(false && "Unexpected decoration for type");
+      CHECK_UNREACHABLE_("Unexpected decoration for type");
     }
   }
 
@@ -354,8 +354,7 @@ string MungedForwardDeclareLine(const NamedDecl* decl) {
     return MungedForwardDeclareLineForNontemplates(rec_decl);
   else if (const TemplateDecl* template_decl = DynCastFrom(decl))
     return MungedForwardDeclareLineForTemplates(template_decl);
-  CHECK_(false && "Unexpected decl type for MungedForwardDeclareLine");
-  return "<error>";
+  CHECK_UNREACHABLE_("Unexpected decl type for MungedForwardDeclareLine");
 }
 
 }  // namespace internal
@@ -365,7 +364,7 @@ OneIncludeOrForwardDeclareLine::OneIncludeOrForwardDeclareLine(
     : line_(internal::MungedForwardDeclareLine(fwd_decl)),
       start_linenum_(-1), end_linenum_(-1),     // set 'for real' below
       is_desired_(false), is_present_(false), symbol_counts_(),
-      quoted_include_(), fwd_decl_(fwd_decl) {
+      quoted_include_(), included_file_(NULL), fwd_decl_(fwd_decl) {
   const SourceRange decl_lines = GetSourceRangeOfClassDecl(fwd_decl);
   // We always want to use the instantiation line numbers: for code like
   //     FORWARD_DECLARE_CLASS(MyClass);
@@ -375,11 +374,12 @@ OneIncludeOrForwardDeclareLine::OneIncludeOrForwardDeclareLine(
 }
 
 OneIncludeOrForwardDeclareLine::OneIncludeOrForwardDeclareLine(
-    const string& quoted_include, int linenum)
+    const FileEntry* included_file, const string& quoted_include, int linenum)
     : line_("#include " + quoted_include),
       start_linenum_(linenum), end_linenum_(linenum),
       is_desired_(false), is_present_(false), symbol_counts_(),
-      quoted_include_(quoted_include), fwd_decl_(NULL) {
+      quoted_include_(quoted_include), included_file_(included_file),
+      fwd_decl_(NULL) {
 }
 
 bool OneIncludeOrForwardDeclareLine::HasSymbolUse(const string& symbol_name)
@@ -410,7 +410,8 @@ IwyuFileInfo::IwyuFileInfo(const clang::FileEntry* this_file,
   : file_(this_file),
     preprocessor_info_(preprocessor_info),
     quoted_file_(quoted_include_name),
-    internal_headers_(),
+    is_prefix_header_(false),
+    associated_headers_(),
     symbol_uses_(),
     lines_(),
     direct_includes_(),
@@ -420,15 +421,16 @@ IwyuFileInfo::IwyuFileInfo(const clang::FileEntry* this_file,
     desired_includes_have_been_calculated_(false)
 {}
 
-void IwyuFileInfo::AddInternalHeader(const IwyuFileInfo* other) {
+void IwyuFileInfo::AddAssociatedHeader(const IwyuFileInfo* other) {
   VERRS(6) << "Adding " << GetFilePath(other->file_)
-           << " as internal header for " << GetFilePath(file_) << "\n";
-  internal_headers_.insert(other);
+           << " as associated header for " << GetFilePath(file_) << "\n";
+  associated_headers_.insert(other);
 }
 
 void IwyuFileInfo::AddInclude(const clang::FileEntry* includee,
                               const string& quoted_includee, int linenumber) {
-  OneIncludeOrForwardDeclareLine new_include(quoted_includee, linenumber);
+  OneIncludeOrForwardDeclareLine new_include(includee, quoted_includee,
+                                             linenumber);
   new_include.set_present();
 
   // It's possible for the same #include to be seen multiple times
@@ -1290,7 +1292,7 @@ void IwyuFileInfo::CalculateIwyuViolations(vector<OneUse>* uses) {
 
   // (C1) Compute the direct includes of 'associated' files.
   set<string> associated_direct_includes;
-  for (Each<const IwyuFileInfo*> it(&internal_headers_); !it.AtEnd(); ++it) {
+  for (Each<const IwyuFileInfo*> it(&associated_headers_); !it.AtEnd(); ++it) {
     ReportIncludeFileUse((*it)->quoted_file_);
     InsertAllInto((*it)->direct_includes(), &associated_direct_includes);
   }
@@ -1315,11 +1317,10 @@ void IwyuFileInfo::CalculateIwyuViolations(vector<OneUse>* uses) {
   // The 'effective' desired includes are defined to be the desired
   // includes of associated, plus us.  These are used to decide if a
   // particular use will be satisfied after fixing the #includes.
-  // NOTE: this depends on our internal headers having had their
+  // NOTE: this depends on our associated headers having had their
   // iwyu analysis done before us.
   set<string> effective_desired_includes = desired_includes();
-  for (Each<const IwyuFileInfo*> it(&internal_headers_); !it.AtEnd(); ++it)
-    InsertAllInto((*it)->desired_includes(), &effective_desired_includes);
+  InsertAllInto(AssociatedDesiredIncludes(), &effective_desired_includes);
 
   // Now that we've figured out desired_includes, figure out iwyu violations.
   for (vector<OneUse>::iterator it = uses->begin(); it != uses->end(); ++it) {
@@ -1400,8 +1401,8 @@ void CalculateDesiredIncludesAndForwardDeclares(
     if (use->is_full_use()) {
       CHECK_(use->has_suggested_header() && "Full uses should have #includes");
       if (!ContainsKey(include_map, use->suggested_header())) {  // must be added
-        lines->push_back(OneIncludeOrForwardDeclareLine(use->suggested_header(),
-                                                        -1));
+        lines->push_back(OneIncludeOrForwardDeclareLine(
+            GetFileEntry(use->decl()), use->suggested_header(), -1));
         include_map[use->suggested_header()] = lines->size() - 1;
       }
       const int index = include_map[use->suggested_header()];
@@ -1452,6 +1453,49 @@ void CalculateDesiredIncludesAndForwardDeclares(
   }
 }
 
+bool IsPrefixHeader(const FileEntry* file_entry,
+                    const IwyuPreprocessorInfo* preprocessor_info) {
+  if (file_entry) {
+    IwyuFileInfo* file_info = preprocessor_info->FileInfoFor(file_entry);
+    if (file_info)
+      return file_info->is_prefix_header();
+  }
+  return false;
+}
+
+void CleanupPrefixHeaderIncludes(
+    const IwyuPreprocessorInfo* preprocessor_info,
+    vector<OneIncludeOrForwardDeclareLine>* lines) {
+  CommandlineFlags::PrefixHeaderIncludePolicy policy =
+      GlobalFlags().prefix_header_include_policy;
+  if (policy == CommandlineFlags::kAdd)
+    return;
+
+  for (vector<OneIncludeOrForwardDeclareLine>::iterator it = lines->begin();
+       it != lines->end(); ++it) {
+    if (!it->is_desired())
+      continue;
+    if (it->is_present() && (policy == CommandlineFlags::kKeep))
+      continue;  // Keep present line according to policy.
+
+    const FileEntry* file_entry = NULL;
+    if (it->IsIncludeLine()) {
+      file_entry = it->included_file();
+      CHECK_(file_entry && "Valid file_entry is expected");
+    } else {
+      const RecordDecl* dfn = GetDefinitionForClass(it->fwd_decl());
+      file_entry = GetFileEntry(dfn);
+    }
+    if (IsPrefixHeader(file_entry, preprocessor_info)) {
+      CHECK_(file_entry && "FileEntry should exist to be prefix header");
+      it->clear_desired();
+      VERRS(6) << "Ignoring '" << it->line()
+               << "': is superseded by command line include "
+               << file_entry->getName() << "\n";
+    }
+  }
+}
+
 // Used by GetSymbolsSortedByFrequency().
 class CountGt {
  public:
@@ -1549,7 +1593,7 @@ LineSortKey GetSortKey(const OneIncludeOrForwardDeclareLine& line,
 }
 
 // filename is "this" filename: the file being emitted.
-// associated_filepaths are the quoted-include form of internal_headers_.
+// associated_filepaths are the quoted-include form of associated_headers_.
 string PrintableDiffs(const string& filename,
                       const set<string>& associated_quoted_includes,
                       const vector<OneIncludeOrForwardDeclareLine>& lines) {
@@ -1634,10 +1678,7 @@ int IwyuFileInfo::CalculateAndReportIwyuViolations() {
   // bar.h and foo.h is adding it, we don't need to add it ourself.
   // On the other hand, if foo.h used to have it but is removing it,
   // we *do* need to add it.
-  set<string> associated_desired_includes;
-  for (Each<const IwyuFileInfo*> it(&internal_headers_); !it.AtEnd(); ++it) {
-    InsertAllInto((*it)->desired_includes(), &associated_desired_includes);
-  }
+  set<string> associated_desired_includes = AssociatedDesiredIncludes();
 
   CalculateIwyuViolations(&symbol_uses_);
   const int retval = EmitWarningMessages(symbol_uses_);
@@ -1654,6 +1695,8 @@ int IwyuFileInfo::CalculateAndReportIwyuViolations() {
     }
   }
 
+  internal::CleanupPrefixHeaderIncludes(preprocessor_info_, &lines_);
+
   EmitDiffs(lines_);
   return retval;
 }
