@@ -481,12 +481,12 @@ void IwyuPreprocessorInfo::AddDirectInclude(
 
   // We have a rule that if foo.h #includes bar.h, foo.cc doesn't need
   // to #include bar.h as well, but instead gets it 'automatically'
-  // via foo.h.  We say that 'foo.h' is an "internal header" for
+  // via foo.h.  We say that 'foo.h' is an "associated header" for
   // foo.cc.  Make sure we ignore self-includes, though!
-  // iwyu_output.cc gets upset if a file is its own internal header.
+  // iwyu_output.cc gets upset if a file is its own associated header.
   if (includer == main_file_ && includee != includer &&
       BelongsToMainCompilationUnit(includer, includee))
-    GetFromFileInfoMap(includer)->AddInternalHeader(
+    GetFromFileInfoMap(includer)->AddAssociatedHeader(
         GetFromFileInfoMap(includee));
 
   // Also keep track of what FileEntry we ended up using for this name.
@@ -564,7 +564,8 @@ void IwyuPreprocessorInfo::MacroDefined(const Token& id,
   }
 }
 
-void IwyuPreprocessorInfo::If(SourceLocation loc, SourceRange condition_range) {
+void IwyuPreprocessorInfo::If(SourceLocation loc, SourceRange condition_range,
+                              bool condition_value) {
   ERRSYM(GetFileEntry(condition_range.getBegin()))
       << " [ #if         ] "
       << PrintableSourceRange(condition_range) << "\n";
@@ -573,6 +574,7 @@ void IwyuPreprocessorInfo::If(SourceLocation loc, SourceRange condition_range) {
 
 void IwyuPreprocessorInfo::Elif(SourceLocation loc,
                                 SourceRange condition_range,
+                                bool condition_value,
                                 SourceLocation if_loc) {
   ERRSYM(GetFileEntry(condition_range.getBegin()))
       << " [ #elif       ] "
@@ -618,20 +620,19 @@ void IwyuPreprocessorInfo::FileChanged(SourceLocation loc,
   switch (reason) {
     case EnterFile:
       FileChanged_EnterFile(loc);
-      break;
+      return;
     case ExitFile:
       FileChanged_ExitToFile(loc, exiting_from);
-      break;
+      return;
     case RenameFile:
       FileChanged_RenameFile(loc);
-      break;
+      return;
     case SystemHeaderPragma:
       // We see "#pragma GCC system_header".
       FileChanged_SystemHeaderPragma(loc);
-      break;
-    default:
-      CHECK_(false && "Unknown file change reason");
+      return;
   }
+  CHECK_UNREACHABLE_("Unknown file change reason");
 }
 
 // Called when we see an #include, but decide we don't need to
@@ -692,6 +693,23 @@ void IwyuPreprocessorInfo::FileChanged_EnterFile(
   if (ShouldReportIWYUViolationsFor(new_file)) {
     files_to_report_iwyu_violations_for_.insert(new_file);
   }
+
+  // Mark is_prefix_header.
+  CHECK_(new_file && "is_prefix_header is applicable to usual files only");
+  IwyuFileInfo *includee_file_info = GetFromFileInfoMap(new_file);
+  const FileEntry* includer_file = GetFileEntry(include_loc);
+  bool is_prefix_header = false;
+  if (includer_file) {
+    // File included from another prefix header file is prefix header too.
+    IwyuFileInfo *includer_file_info = GetFromFileInfoMap(includer_file);
+    is_prefix_header = includer_file_info->is_prefix_header();
+  } else {
+    // Files included from command line are prefix headers, unless it's the
+    // main file.
+    is_prefix_header = (new_file != main_file_);
+  }
+  if (is_prefix_header)
+    includee_file_info->set_prefix_header();
 }
 
 // Called when done with an #included file and returning to the parent file.
@@ -814,7 +832,7 @@ void IwyuPreprocessorInfo::PopulateIntendsToProvideMap() {
   // direct includes), you bring in all its includes as well.
   // Basically, a public header is really an equivalence class of
   // itself and all its direct includes.
-  // TODO(csilvers): use AddInternalHeaders() to get includes here.
+  // TODO(csilvers): use AddAssociatedHeaders() to get includes here.
   for (Each<const FileEntry*, IwyuFileInfo> it(&iwyu_file_info_map_);
        !it.AtEnd(); ++it) {
     const FileEntry* file = it->first;
